#!/usr/bin/env python3

# this is a script for generating icon files that Mac (icns) and Windows (ico)
# use
#
# guide:
#
# - open raw icon SVG in editor (e.g. inkscape)
# - save the raw icon SVG as a high (e.g. 1024x1024) resolution PNG file (e.g. logoBIG.png)
# - input the PNG file into this script, and tell the script what format to write the output to:
#
#     - e.g. (icns) `python3 generate_icon_files.py logoBIG.png -o icon.icns`
#     - e.g. (ico)  `python3 generate_icon_files.py logoBIG.png -o icon.ico`
#
# ICO files generated by this script embed the following sizes:
#
#     [(16, 16), (24, 24), (32, 32), (48, 48), (64, 64), (128, 128), (255, 255)]
#
# (see: https://stackoverflow.com/questions/45507/is-there-a-python-library-for-generating-ico-files)

import argparse
import os
from PIL import Image  # pip install Pillow

def convert(input_filename, output_filename):
    assert os.path.exists(input_filename)

    img = Image.open(input_filename)
    img.save(output_filename)

def main():
    parser = argparse.ArgumentParser(description="Convert image using pillow")
    parser.add_argument("input", help="Input File")
    parser.add_argument("-o", required=True, help="Ouput File")
    args = parser.parse_args()
    convert(args.input, args.o)

if __name__ == "__main__":
    main()